/*
  * Copyright (c) 2019 Ignacio Francisco Ramírez Paulino and Ignacio Hounie
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Affero General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or (at
  * your option) any later version.
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
  * General Public License for more details.
  * You should have received a copy of the GNU Affero General Public License
  *  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * \file paco_admm.h
 * \brief ADMM interface
 */
#ifndef PACO_ALGORITHMS_H
#define PACO_ALGORITHMS_H

#include "paco_types.h"
#include "paco_iterate.h"

/**
 * Optimization parameters for the PACO-ADMM problem.
 */
typedef struct _admm_params_ {
    double lambda0; ///< inverse quadratic penalty  constant in the Augmented Lagrangian: (1/2lambda)||constraint_violation||2
    double min_cost_change; ///< if the relative cost value changes less than this number then the algorithm is considered to have reached its optimum
    double min_arg_change; ///< if the relative argument norm value changes less than this number then the algorithm is considered to have reached its optimum
    double kappa; ///< lambda will be multipled by this value each time the cost function increases instead of decreasing
    index_t max_iter; ///< maximum number of iterations
} admm_params_st;

/**
 * Given an instance of a PACO problem and a set of optimization parameters,
 * solve the problem using the ADMM method. (ALGORITHM 1 in the paper).
 */
void paco_admm(paco_iterate_st* iterate, const admm_params_st* params);

#endif