/*
  * Copyright (c) 2019 Ignacio Francisco Ramírez Paulino and Ignacio Hounie
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Affero General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or (at
  * your option) any later version.
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
  * General Public License for more details.
  * You should have received a copy of the GNU Affero General Public License
  *  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PACO_ITERATE_H
#define PACO_ITERATE_H
/**
 * \file paco_iterate.h
 * \brief Stores the variables at each iterate of the ADMM algorithm
 */
#include "paco_types.h"

/**
 *  Stores the variables at each iterate of the ADMM algorithm
 */
typedef struct _paco_iterate_ {
    index_t k; ///< iteration number
    double lambda; ///< penalty parameter for this iteration
    gsl_matrix_float* A; ///< main variable
    gsl_matrix_float* B; ///< mirror (split) variable
    gsl_matrix_float* U; ///< (Scaled) Lagrange multiplier
    gsl_matrix_float* Y; ///< intermediate algorithm output
    double nA; ///< Frobenius norm of A
    double nB; ///< Frobenius norm of B
    double nU; ///< Frobenius norm of U
    double dAB; ///< ||A-B||
    double f; ///< problem cost function
} paco_iterate_st;

/** 
 * \brief allocate space for the iterate.
 * DOES NOT INTIALIZE VALUES! Initialization is problem specific
 */
paco_iterate_st* paco_iterate_alloc();

/**
 * frees up space taken by the iterate 
 */
void paco_iterate_free(paco_iterate_st*);

#endif