/*
  * Copyright (c) 2019 Ignacio Francisco Ramírez Paulino and Ignacio Hounie
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Affero General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or (at
  * your option) any later version.
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
  * General Public License for more details.
  * You should have received a copy of the GNU Affero General Public License
  *  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * \file paco_linop.h
 *\brief Interface to patch models based on linear operators.
 */
#ifndef LINOP_H
#define LINOP_H

/**
 * \brief Applies the 2D orthogonal DCT Type II direct transform to each row of X (n x m)
 * which is treated as an R^{w1 x w2} matrix, where w1 * w2 = m
 */
void dct2d(gsl_matrix_float* TX, const index_t w1, const index_t w2, const gsl_matrix_float* X);

/**
 * \brief Applies the 2D  orthogonal INVERSE DCT Type II direct transform to each row of X (n x m)
 * which is treated as an R^{w1 x w2} matrix, where w1 * w2 = m
 */
void idct2d(gsl_matrix_float* X, const index_t w1, const index_t w2, const gsl_matrix_float* TX);

/**
 * \brief Applies the NORMALIZED 2D DCT Type II direct transform to each row of X (n x m)
 * which is treated as an R^{w1 x w2} matrix, where w1 * w2 = m
 */
void dct2dn(gsl_matrix_float* TX, const index_t w1, const index_t w2, const gsl_matrix_float* X);

/**
 * \brief Applies the NORMALIZED 2D  INVERSE DCT Type II direct transform to each row of X (n x m)
 * which is treated as an R^{w1 x w2} matrix, where w1 * w2 = m
 */
void idct2dn(gsl_matrix_float* X, const index_t w1, const index_t w2, const gsl_matrix_float* TX);


#endif
